
# q-expansion of newform 245.4.a.j, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 245
weight = 4
poly_data = [-11, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, 1], [5, 0], [5, 0], [0, 0], [33, -4], [5, 20], [35, -20], [70, 20], [-8, -28], [-129, -48], [-10, -60], [164, 44], [150, -100], [-58, 12], [-15, -40], [270, -120], [190, 40], [540, -60], [234, -96], [-528, 64], [-430, -200], [79, 348], [20, -200], [-120, 380], [815, 180], [-1010, 140], [-15, 240], [-90, -160], [-951, 288], [1188, 68], [-1834, -64], [-120, 380], [-1220, 220], [2800, 0], [-134, 392], [-513, 444], [-490, -160], [-2760, 360], [1125, -680], [2875, -100], [116, -400], [-1970, -320], [153, -700], [2512, -308], [714, 804], [4330, 80], [-4657, -100], [1405, 720], [1125, 840], [-110, 140], [2088, 1328], [-2001, 500], [2380, -20], [540, 1040], [1230, 320], [1558, 1448], [-1920, 280], [-1060, 540], [-620, -880], [723, -1624], [1875, -1200], [-5865, -220], [-285, -720], [-3310, 740], [-2355, -300], [6386, 376], [-748, -776], [-1040, -360], [3444, -276], [1490, -60], [-5845, -580], [-2576, 88], [11105, -120], [2128, -892], [3104, 648], [-4080, -2160], [-7419, -328], [-4505, 2300], [-5763, -1656], [-1950, -1800], [-1410, -1160], [-1647, 936], [-3683, -3500], [-5650, 1760], [-2780, 1920], [3390, 460], [5791, 392], [-4646, 2924], [-15620, -320], [6880, 2620], [4655, -400], [-1540, -3940], [6956, -2664], [8143, -1484], [-5211, 4500], [1415, 200], [-3320, -4720], [9630, 2380], [8320, 2080], [-4893, -3216], [-6316, -716], [-7164, -64], [18580, -1720], [-16814, 1432], [10028, 4136], [-16015, 1860], [10340, -80], [-555, -2060], [-4359, 1692], [-13210, 3440], [6875, -1320], [-17522, -672], [-396, 5184], [1540, 7540], [-323, 1724], [13782, -3416], [5085, 600], [6260, -880], [-18750, -1300], [-579, 700], [-1300, -3400], [5040, -4740], [-13175, -4940], [7292, 892], [-11660, 840], [-11473, -3200], [-17929, -3000], [-3920, 8980], [-560, 8960], [15185, -1260], [-6131, -9372], [1530, 2920], [1327, 1700], [-7864, -3464], [26480, 1780], [-14210, -3360], [3365, 6860], [20545, -6880], [10035, -2140], [-2699, -488], [10440, -2760], [15737, 3200], [-1682, 3468], [-2856, -8296], [-29690, -340], [3180, -8420], [-16190, 1320], [-14950, -2560], [28580, -5920], [-6198, 7692], [6750, -5700], [-8960, -6160], [21652, -5268], [3120, 5640], [9620, -5420], [-9408, 1504], [-14489, -7468], [-3450, -3000], [8635, 5700], [24380, -3020], [10410, -5060], [25462, -7388], [-28430, -3520], [-12610, 7540], [8646, 10956], [-15325, 2240], [-468, 16584], [-10605, 1380]]
